@@ -88,9 +88,6 @@ static int wcmAllocate(InputInfoPtr pInfo)
 	priv->strip_default[STRIP_RIGHT_DN] = 5;
 	priv->naxes = 6;			/* Default number of axes */
 
-	/* JEJ - throttle sampling code */
-	priv->throttleLimit = -1;
-
 	common->wcmDevices = priv;
 
 	/* tool */
@@ -173,7 +170,7 @@ wcmSetType(InputInfoPtr pInfo, const char *type)
 		goto invalid;
 
 	/* Set the device id of the "last seen" device on this tool */
-	priv->old_device_id = wcmGetPhyDeviceID(priv);
+	priv->oldState.device_id = wcmGetPhyDeviceID(priv);
 
 	if (!priv->tool)
 		return 0;
@@ -239,6 +236,9 @@ static void wcmUninit(InputDriverPtr drv, InputInfoPtr pInfo, int flags)
 
 	DBG(1, priv, "\n");
 
+	if (WACOM_DRIVER.active == priv)
+		WACOM_DRIVER.active = NULL;
+
 	/* Server 1.10 will UnInit all devices for us */
 #if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 12
 	if (priv->isParent)
@@ -320,10 +320,19 @@ static void wcmSplitName(char* devicename, char *basename, char *subdevice, char
 	{
 		*a = '\0';
 		b = strrchr(name, ' ');
-		if (b && (!strcmp(b, " Pen") || !strcmp(b, " Finger") || !strcmp(b, " Pad")))
+
+		while (b)
 		{
-			*b = '\0';
-			strncat(subdevice, b+1, len-1);
+			if (!strcmp(b, " Pen") || !strcmp(b, " Finger") ||
+			    !strcmp(b, " Pad") || !strcmp(b, " Touch"))
+			{
+				*b = '\0';
+				strncpy(subdevice, b+1, len-1);
+				subdevice[len-1] = '\0';
+				b = strrchr(name, ' ');
+			}
+			else
+				b = NULL;
 		}
 		strncat(tool, a+1, len-1);
 	}
@@ -458,7 +467,7 @@ wcmInitModel(InputInfoPtr pInfo)
 	float version;
 
 	/* Initialize the tablet */
-	if(common->wcmDevCls->Init(pInfo, id, &version) != Success ||
+	if(common->wcmDevCls->Init(pInfo, id, ARRAY_SIZE(id), &version) != Success ||
 		wcmInitTablet(pInfo, id, version) != Success)
 		return FALSE;
 
@@ -698,7 +707,7 @@ SetupProc_fail:
 	return BadMatch;
 }
 
-InputDriverRec WACOM =
+static InputDriverRec WACOM =
 {
 	1,             /* driver version */
 	"wacom",       /* driver name */
